"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = manager;

var _difference2 = _interopRequireDefault(require("lodash/difference"));

var _actions = require("./actions");

var _reducer = require("./reducer");

var _listeners = require("./listeners");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Registers redux-scripts-manager with the store
 * @param store Redux Store
 * @param key Reducer Key in Store
 */
function manager(store, key = 'scripts') {
  let prevState = _reducer.initialState;
  store.subscribe(() => {
    const state = store.getState()[key];
    if (state === prevState) return;
    const diff = (0, _difference2.default)(state.loading, prevState.loading);
    prevState = state;
    if (!diff.length) return;
    const head = document.querySelector('head');
    diff.forEach(src => {
      const script = document.createElement('script');
      script.src = src;
      script.type = 'text/javascript';
      script.async = true;
      const callbackName = state.callbacks[src];

      if (callbackName) {
        window[callbackName] = () => {
          store.dispatch((0, _actions.scriptLoaded)(src));
          (0, _listeners.notify)(src);
          delete window[callbackName];
        };
      } else {
        script.onload = () => {
          store.dispatch((0, _actions.scriptLoaded)(src));
          (0, _listeners.notify)(src);
        };
      }

      head.appendChild(script);
    });
  });
}
//# sourceMappingURL=manager.js.map